/**
 * 小程序
 * 党员基本信息
 */

import * as partyMemberData from "../../../data/partyMember";
import * as configEnum from "../../../config/enum";
import * as verificationEnumTools from "../../../util/verificationEnum";
import { eccFormParam } from "../../../util/verificationParam";
import * as eccFormParamConfig from "../../../config/eccFormParamConfig";
import { getPwdMd5, getToken } from "../../../tools/system";
import { extractData } from "../../../util/piecemeal";
import * as splitResultConfig from "../../../config/splitResultConfig";
import moment = require("moment");
import { BizError } from "../../../util/bizError";
import { ERRORENUM } from "../../../config/errorEnum";


/**
 * 获取党员名册
 * @param branchName 支部id
 * @returns 
 */
export async function getPartyMemberName(branchName) {
    let partyBranchName = verificationEnumTools.changeEnumValue(configEnum.BRANCHNAMETYPE, branchName);
    let partyMemberNum = await partyMemberData.groupFindPartyMemberCount({branchName:{$in: branchName }}); //党员总数

    let partyMemberCount = {
        key: partyBranchName + "目前党员人数",
        value: partyMemberNum
    };

    let partyMemberList = await partyMemberData.findPartyMemberAll({branchName:{$in: branchName }});
    let nameList = [];
    partyMemberList.forEach( info => {
        let changeData:any = extractData(splitResultConfig.partyMemberConfig, info, false);
        switch (changeData.entryType) {
            case 1:
                changeData.color = "#C62A23";
                changeData.partyPositions = verificationEnumTools.changeEnumValue(configEnum.PARTYPOSITIONSICON, info.partyPositions);
            break;
            case 2:
                changeData.color = "#3549B0";
                changeData.partyPositions = verificationEnumTools.changeEnumValue(configEnum.ENTRYTYPE, info.entryType);
            break;
            case 3:
                changeData.color = "#E4B386";
                changeData.partyPositions = verificationEnumTools.changeEnumValue(configEnum.ENTRYTYPE, info.entryType);
            break;
        }
        
        let time = moment(new Date()).format("YYYY");
        let partyJoiningTime = moment(info.partyJoiningTime).format("YYYY");
        changeData.partyStanding = parseInt(time) - parseInt(partyJoiningTime) + "年";
        changeData.department = verificationEnumTools.changeEnumValue(configEnum.DEDPARTMENT, changeData.department);
        nameList.push(changeData);
    })

    return {nameList, partyMemberCount};
}



/**
 * 创建党员基础信息
 * @param param  {name: "姓名", sex: 性别, birthDate: 出生年月, phone: "手机号码", levelOfEducation: "文化程度", partyJoiningTime: 入党时间, administrativePosition: 行政职务, 
 *                remarks: "备注", entryType: 名册录入类型, partyPositions: [党内职务, ...], listedAsActivistsTime: 列为积极份子时间, talkTime: 谈话时间, loginId: "登录账号"}
 * @return {isSuccess: true}
 */
 export async function createPartyMember(param, entryType) {
    eccFormParam("创建正式党员录入", eccFormParamConfig.CreatePartyMemberFormConfig, param);
    verificationEnumTools.eccEnumValue('正式党员录入性别', 'partyActivityType', configEnum.SEX, param.partyActivityType);
    verificationEnumTools.eccEnumValue('正式党员录入党内职务图例', 'partyPositionsIcon', configEnum.PARTYPOSITIONSICON, param.partyPositions);
    verificationEnumTools.eccEnumValue('正式党员录入党内职务', 'partyPositions', configEnum.PARTYPOSITIONS, param.partyPositions);
    verificationEnumTools.eccEnumValue('正式党员名册录入类型', 'entryType', configEnum.ENTRYTYPE, param.entryType);

    if (entryType) param.entryType = entryType;

    let partyMemberInfo:any = await partyMemberData.createPartyMember(param);

    partyMemberInfo.loginId = param.phone;
    if(!param.pwd) partyMemberInfo.pwd = getPwdMd5(param.phone, param.phone.slice(5, 11));
    else partyMemberInfo.pwd = getPwdMd5(param.phone, param.pwd);
    partyMemberInfo.token = getToken(param.phone);
    partyMemberInfo.tokenMs = new Date().valueOf();

    await partyMemberInfo.save();

    return {isSuccess: true};
}


/**
 * 党员详情
 * @param id 党员id
 * @return 
 */
export async function partyMemberDetails(id:number) {
    let partyMemberList = await partyMemberData.findPartyMemberById(id);
    let changeData:any = {};
    partyMemberList.forEach( info => {
        changeData = extractData(splitResultConfig.partyMemberDetailsConfig, info, false);
        switch (changeData.entryType) {
            case 1:
                changeData.color = "#C62A23";
                changeData.partyPositions = verificationEnumTools.changeEnumValue(configEnum.PARTYPOSITIONSICON, info.partyPositions);
            break;
            case 2:
                changeData.color = "#3549B0";
                changeData.partyPositions = verificationEnumTools.changeEnumValue(configEnum.ENTRYTYPE, info.entryType);
            break;
            case 3:
                changeData.color = "#E4B386";
                changeData.partyPositions = verificationEnumTools.changeEnumValue(configEnum.ENTRYTYPE, info.entryType);
            break;
        }
        changeData.department = verificationEnumTools.changeEnumValue(configEnum.DEDPARTMENT, info.department);
        changeData.sex = verificationEnumTools.changeEnumValue(configEnum.SEX, info.sex);
        changeData.administrativePosition = verificationEnumTools.changeEnumValue(configEnum.ADMINISTRATIVEPOSITION, info.administrativePosition);
        changeData.partyState = verificationEnumTools.changeEnumValue(configEnum.PARTYSTATE, info.partyState);
        
        changeData.birthDate = moment(info.birthDate).format("YYYY年MM月DD日");
        changeData.partyJoiningTime = moment(info.partyJoiningTime).format("YYYY年MM月DD日");
        changeData.study = [
            {key:"学习排名", value:getValueByLength(1)},
            {key:"学习课时", value:getValueByLength(2)},
            {key:"学习进度", value:getValueByLength(2) + "%"}
        ];

        let time = moment(new Date()).format("YYYY");
        let partyJoiningTime = moment(info.partyJoiningTime).format("YYYY");
        changeData.partyStanding = parseInt(time) - parseInt(partyJoiningTime) + "年";
        
    })

    return changeData;
}


/**
 * 删除党员
 * @param id 党员id
 * @returns {isSuccess:true}
 */
 export async function deletePartyMember(id:number) {
    let activityInfo = await partyMemberData.findPartyMemberById(id);
    if (!activityInfo) throw new BizError(ERRORENUM.未找到数据, `库中不存在id为${id}党员信息`);

    await partyMemberData.deletePartyMember(id);

    return {isSuccess:true};
}







/**
 * 随机数生成假数据
 * @param length 假数据长度
 * @returns 
 */
function getValueByLength(length) {
    let i = 1;
    getPlaByLength(length, () => {
        i= i * 10;
    });
    return Math.floor(Math.random() * i);
}

function getPlaByLength(num, callback) {
    for (let i =0; i < num; i++) {
        callback();
    }
}
